@@ -36,7 +36,11 @@ do_test (SoupURI *base_uri, const char *path,
 	GPid pid;
 	gboolean done;
 
-	uri = soup_uri_new_with_base (base_uri, path);
+	/* We build the URI this way to avoid having soup_uri_new()
+	   normalize the path, hence losing the encoded characters in
+	   tests 4. and 5. below. */
+	uri = soup_uri_copy (base_uri);
+	soup_uri_set_path (uri, path);
 	uri_str = soup_uri_to_string (uri, FALSE);
 	soup_uri_free (uri);
 
@@ -156,7 +160,46 @@ do_server_auth_test (gconstpointer data)
 		 /* success? */
 		 TEST_USES_DIGEST (i) && TEST_GOOD_AUTH (i));
 
-	/* 4. Any auth required. */
+	/* 4. Digest auth with encoded URI. See #794208.
+	 */
+	do_test (base_uri, "/Digest/A%20B",
+		 TEST_GOOD_USER (i), TEST_GOOD_PASSWORD (i),
+		 /* request */
+		 TEST_USES_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from client */
+		 TEST_PREEMPTIVE_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from server */
+		 FALSE, TRUE,
+		 /* success? */
+		 TEST_USES_DIGEST (i) && TEST_GOOD_AUTH (i));
+
+	/* 5. Digest auth with a mixture of encoded and decoded chars in the URI. See #794208.
+	 */
+	do_test (base_uri, "/Digest/A%20|%20B",
+		 TEST_GOOD_USER (i), TEST_GOOD_PASSWORD (i),
+		 /* request */
+		 TEST_USES_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from client */
+		 TEST_PREEMPTIVE_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from server */
+		 FALSE, TRUE,
+		 /* success? */
+		 TEST_USES_DIGEST (i) && TEST_GOOD_AUTH (i));
+
+	/* 6. Digest auth with UTF-8 chars in the URI. See #794208.
+	 */
+	do_test (base_uri, "/Digest/A௹B",
+		 TEST_GOOD_USER (i), TEST_GOOD_PASSWORD (i),
+		 /* request */
+		 TEST_USES_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from client */
+		 TEST_PREEMPTIVE_BASIC (i), TEST_USES_DIGEST (i),
+		 /* expected from server */
+		 FALSE, TRUE,
+		 /* success? */
+		 TEST_USES_DIGEST (i) && TEST_GOOD_AUTH (i));
+
+	/* 7. Any auth required. */
 	do_test (base_uri, "/Any/foo",
 		 TEST_GOOD_USER (i), TEST_GOOD_PASSWORD (i),
 		 /* request */
@@ -168,7 +211,7 @@ do_server_auth_test (gconstpointer data)
 		 /* success? */
 		 (TEST_USES_BASIC (i) || TEST_USES_DIGEST (i)) && TEST_GOOD_AUTH (i));
 
-	/* 5. No auth required again. (Makes sure that
+	/* 8. No auth required again. (Makes sure that
 	 * SOUP_AUTH_DOMAIN_REMOVE_PATH works.)
 	 */
 	do_test (base_uri, "/Any/Not/foo",
@@ -282,7 +325,7 @@ main (int argc, char **argv)
 
 	test_init (argc, argv, no_test_entry);
 
-	server = soup_test_server_new (FALSE);
+	server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
 	g_signal_connect (server, "request_started",
 			  G_CALLBACK (request_started_callback), NULL);
 	soup_server_add_handler (server, NULL,
@@ -310,12 +353,10 @@ main (int argc, char **argv)
 
 	loop = g_main_loop_new (NULL, TRUE);
 
+	base_uri = soup_test_server_get_uri (server, "http", NULL);
 	if (run_tests) {
 		int i;
 
-		base_uri = soup_uri_new ("http://127.0.0.1");
-		soup_uri_set_port (base_uri, soup_server_get_port (server));
-
 		for (i = 0; i < 16; i++) {
 			char *path;
 			const char *authtypes;
@@ -344,13 +385,12 @@ main (int argc, char **argv)
 		}
 
 		ret = g_test_run ();
-
-		soup_uri_free (base_uri);
 	} else {
-		g_print ("Listening on port %d\n", soup_server_get_port (server));
+		g_print ("Listening on port %d\n", base_uri->port);
 		g_main_loop_run (loop);
 		ret = 0;
 	}
+	soup_uri_free (base_uri);
 
 	g_main_loop_unref (loop);
 	soup_test_server_quit_unref (server);
